/*********************************************************************
 * File:   ttycmds.c  (part of TTYTALK)
 * Author: Victor Volkman
 * Date:   June, 1988
 ********************************************************************/
#include "tty.h"

#define CURSOR_OFFS 10

char *term_cmds[] = {
    "ANswback","ATten","BKsize","BLankex","BReak",   /* Config commands */
    "CApture","CWait","DAta","DEbug","DPrefix",
    "DSuffix","DUplex","FIlter","INfilter","LBreak",
    "LFauto","LWait","NAme","NUmber","OUtfiltr",
    "PArity","POrt","PRinter","RDials","SPeed",
    "STop","TAbex","TUrnarnd","UConly","VIdeo",
    "SEnd","QUit","GO","RXmodem","XXmodem","LOad",
    "SAve"
};

char *parity_types[] = {"None","Odd","Even"};
#define NUM_PARITY (sizeof(parity_types)/sizeof(char *))

char *empty = "";
char *off_on[] = {"Off","On"};

short serial_rates[] = {110,150,300,600,1200,2400,4800,9600};
#define NUM_RATES (sizeof(serial_rates)/sizeof(int))

char *term_strs[NUM_CMDS];
int term_vals[NUM_CMDS];
struct serial_def serial = {0,0,0,0,2};

extern FILE *send_file;
extern FILE *capture_file;

extern int c_row, c_col;
extern enum cmdtype mode;
extern int capture_mode;

void line_cmd()
{
    char cmd_str[80];
    int done_cmd=FALSE;

    while (!done_cmd && mode==CMD_NONE) {
        get_line_cmd(cmd_str);
        if (strlen(cmd_str))
            parse_cmdline(cmd_str);
        else
            done_cmd = TRUE;
    }

    disp_cmdline();
}

void read_cfg(fname)
char *fname;
{
    FILE *infile;
    char fline[80];
    char *more;

    printf("reading %s\n",fname);
    if ((infile=fopen(fname,"r")) == NULL) {
        term_error(1,"Unable to open file for read ",fname);
        return;
    }

    more = fline;
    while (more!=NULL) { /* read and process each input line */
        if ((more=fgets(fline, sizeof(fline), infile)) != NULL) {
            trim_cr(fline);
            parse_cmdline(fline);
        }
    }
    fclose(infile);
}

void get_line_cmd(cmd_str)
char *cmd_str;
{
    char inchar;
    int len=0;

    vid_write_str("                                         ",
     WINDOW_ROWS,0,ATTR_CMD);
    vid_write_str(" Command? _",WINDOW_ROWS,0,ATTR_CMD);
    while ((inchar=kbinput()) != CHAR_CR) {
        if (inchar==CHAR_BS) {
            vid_write(CHAR_BLANK,WINDOW_ROWS,CURSOR_OFFS+len,ATTR_CMD);
            if (len) cmd_str[--len] = CHAR_BLANK;
        }
        else if (len < WINDOW_COLS-CURSOR_OFFS) {
            vid_write(inchar,WINDOW_ROWS,CURSOR_OFFS+len,ATTR_CMD);
            cmd_str[len++] = inchar;
        }
        vid_write(CHAR_UNDER,WINDOW_ROWS,CURSOR_OFFS+len,ATTR_CMD);
        vid_setrc(c_row,c_col);
    }
    cmd_str[len] = '\0';  /* cut here */
}

int parse_cmdline(line)
char *line;
{
    int found;
    int cmd;

    found = FALSE;
    for (cmd=0; cmd<=NUM_CMDS && !found; cmd++)
        found = strnicmp(line, term_cmds[cmd], 2) == 0;
    if (found) {
        cmd--;
        if (term_strs[cmd] != NULL) free(term_strs[cmd]);
        term_strs[cmd] = dup_string(get_parm(line));
        set_parm(cmd);
    }
    else
        term_error(1,"Unknown directive",line);
    return cmd;
}

char *get_parm(val_line)
char *val_line;
{
    char *look;

    if ((look=strchr(val_line,' ')) == NULL)
        return empty;
    if ((look=strrchr(look,' ')) == NULL)
        return empty;
    return ++look;
}

void set_parm(parm_num)
int parm_num;
{
    char *cmd_str;
    short i;
    short cmd_val;
    short found=FALSE;

    cmd_str = term_strs[parm_num];
#if defined MSC
    cmd_val = atoi(cmd_str);
#else
    stcd_i(cmd_str,&cmd_val);
#endif

    switch(parm_num) {
        case CMD_ANSWBACK:
        case CMD_BLANKEX:
        case CMD_DEBUG:
        case CMD_INFILTER:     /* String value:  must be ON or OFF */
        case CMD_LFAUTO:
        case CMD_OUTFILTR:
        case CMD_PRINTER:
        case CMD_TABEX:
        case CMD_UCONLY:
            if (stricmp(cmd_str,off_on[0])==0)
                term_vals[parm_num] = FALSE;
            else if (stricmp(cmd_str,off_on[1])==0)
                term_vals[parm_num] = TRUE;
            else
                term_error(2,"Expected ON or OFF, not ",cmd_str);
            break;

        case CMD_BKSIZE:       /* Numeric parameter:  use # */
        case CMD_CWAIT:
        case CMD_LBREAK:
        case CMD_LWAIT:
        case CMD_RDIALS:
            term_vals[parm_num] = cmd_val;
            break;

        case CMD_ATTEN:
        case CMD_BREAK:
        case CMD_TURNARND:
            break;


        case CMD_DATA:
            stop_rcv();
            if (cmd_val==7 || cmd_val==8)
                serial.data = cmd_val - 5;
            else
                term_error(2,"Illegal word length",cmd_str);
            start_rcv();
            break;

        case CMD_DPREFIX:   /* String value:  no action taken */
        case CMD_DSUFFIX:
        case CMD_NAME:
        case CMD_NUMBER:
            break;

        case CMD_FILTER:
        case CMD_DUPLEX:
            break;

        case CMD_PARITY:
            stop_rcv();
            for (i=0; i<NUM_PARITY && !found; i++)
                found = toupper(cmd_str[0]) == parity_types[i][0];
            if (found) {
                serial.parity = i-1;
                if (serial.parity == 2) /** handle even parity separately */
                    serial.parity = 3;
            }
            else /**/
                term_error(2,"Illegal parity setting",cmd_str);
            start_rcv();
            break;

        case CMD_PORT:
            stop_rcv();
            if (cmd_val==1 || cmd_val==2)
                serial.adapter = cmd_val - 1;
            else
                term_error(2,"Illegal comm port",cmd_str);
            start_rcv();
            break;

        case CMD_SPEED:
            stop_rcv();
            for (i=0; i<NUM_RATES && !found; i++)
                found = cmd_val==serial_rates[i];
            if (found)
                serial.ibaud_rate = i - 1;
            else
                term_error(2,"Illegal baud rate",cmd_str);
            start_rcv();
            break;

        case CMD_STOP:
            stop_rcv();
            if (cmd_val==1 || cmd_val==2)
                serial.stopbits = cmd_val - 1;
            else
                term_error(2,"Illegal stop bits",cmd_str);
            start_rcv();
            break;

        case CMD_SEND:
            if ((send_file=fopen(cmd_str,"r")) != NULL)
                mode = CMD_SEND;
            else
                term_error(2,"Unable to send file",cmd_str);
            break;

        case CMD_CAPTURE:
            if (stricmp(cmd_str,off_on[0])==0) { /* OFF? */
                capture_mode = FALSE;
                if (capture_file!=NULL) fclose(capture_file);
            }
            else if ((capture_file=fopen(cmd_str,"w")) != NULL)
                capture_mode = TRUE;
            else
                term_error(2,"Unable to capture file",cmd_str);
            break;

        case CMD_LOAD:
            read_cfg(cmd_str);
            break;

        case CMD_SAVE:
            write_cfg(cmd_str);
            break;

        case CMD_QUIT:
            mode = CMD_QUIT;
            break;

        case CMD_GO:
            send_dial_string(term_strs[CMD_DPREFIX]);
            send_dial_string(term_strs[CMD_NUMBER]);
            send_dial_string(term_strs[CMD_DSUFFIX]);
            mode = CMD_GO;
            break;
    }
}

void write_cfg(fname)
char *fname;
{
    FILE *outfile;

    if ((outfile=fopen(fname,"w")) == NULL) {
        term_error(1,"Unable to open file for write ",fname);
        return;
    }

    fclose(outfile);
}

char *dup_string(string)
char *string;
{
    char *new_str;

    if ((new_str=calloc(1,strlen(string)+1)) == NULL) {
        term_error(2,"Out of memory",NULL);
        return NULL;
    }
    else
        strcpy(new_str,string);
    return new_str;
}


